
# q-expansion of newform 256.6.b.d, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 256
weight = 6
poly_data = [1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = [[255, [1, 0]], [5, [-1, 0]]]
aps_data = [[0, 0], [0, 10], [0, -37], [-24, 0], [0, -62], [0, -239], [-1198, 0], [0, 1522], [184, 0], [0, 1641], [5728, 0], [0, 5163], [8886, 0], [0, 4594], [-23664, 0], [0, 5843], [0, -8438], [0, 9241], [0, -7766], [-31960, 0], [4886, 0], [-44560, 0], [0, 33682], [-71994, 0], [48866, 0], [0, 25803], [180424, 0], [0, 32850], [0, 56353], [-23502, 0], [94592, 0], [0, 35146], [-277290, 0], [0, 65154], [0, -200765], [-75976, 0], [0, 197161], [0, -5862], [-551928, 0], [0, -216447], [0, 279810], [0, 302355], [409152, 0], [540866, 0], [0, -314949], [283048, 0], [0, 71378], [-889696, 0], [0, 571578], [0, -347893], [347126, 0], [1642960, 0], [-1167438, 0], [0, 395306], [-129790, 0], [70888, 0], [0, -895087], [1773616, 0], [0, -137725], [-594170, 0], [0, -546214], [0, 166827], [0, 529986], [-1336488, 0], [-1644186, 0], [0, 861849], [0, -1374814], [-3414894, 0], [0, -365382], [0, 1148745], [-1170718, 0], [3886536, 0], [-933040, 0], [0, -196109], [0, 2364650], [-1897344, 0], [0, -1861477], [0, -1669039], [4274898, 0], [2573190, 0], [0, 2634138], [0, -486677], [-3557360, 0], [-1954958, 0], [-3296808, 0], [0, 2529098], [2127298, 0], [-289130, 0], [0, -1334351], [-7586192, 0], [0, -709806], [1884064, 0], [-6013880, 0], [0, -2146158], [0, 672546], [202008, 0], [0, -4891719], [10483030, 0], [0, -3105086], [0, -2540439], [0, 1673434], [0, -3501887], [0, -6490942], [-1899418, 0], [0, 831498], [8773442, 0], [0, -2591966], [8498578, 0], [11247096, 0], [3464390, 0], [999712, 0], [0, 4906699], [5347446, 0], [0, 3413842], [-3609704, 0], [-13385342, 0], [0, -4955574], [-17835928, 0], [0, 2161617], [0, 9892914], [0, 7888579], [6787618, 0], [0, -7497109], [0, -5779022], [0, -110078], [0, -2394663], [0, 2134459], [16195952, 0], [6534264, 0], [0, -6580855], [0, -7117382], [-21583544, 0], [-18659376, 0], [0, -1283405], [25958566, 0], [5532674, 0], [0, 4164699], [0, -6826126], [0, 4273129], [-7584842, 0], [0, -3092366], [0, -1390509], [16389464, 0], [0, -11475574], [0, 1750681], [5296680, 0], [0, -10147421], [4827846, 0], [0, 6510522], [39238688, 0], [0, -6731103], [-917710, 0], [0, 12274402], [-16146280, 0], [0, 10168066], [-10772560, 0], [41856568, 0], [29984546, 0], [-14240202, 0], [0, 20727297], [0, -7703934], [20632806, 0], [11872424, 0], [0, -766110], [17432082, 0], [2232696, 0], [-22250080, 0], [0, 26633099]]
